// ==========================================================================
// Base layout
// ==========================================================================

// BORDER-BOX ALL THE THINGS! 
// http://paulirish.com/2012/box-sizing-border-box-ftw/
*, *::after, *::before {  
    box-sizing: border-box; 
}

// Hidden
[hidden] {
    display: none;
}

// Base
html {
    height: 100%;
    background: @body-background fixed;
}
body {
    margin: 0;
    padding: (@padding-base / 2);
}

// Header
header {
    padding: @padding-base;
    margin-bottom: @padding-base;

    p {
        .font-size(18);
    }
    @media (min-width: @screen-sm) {
        padding-top: (@padding-base * 3);
        padding-bottom: (@padding-base * 3);
    }
}

// Sections
section {
    max-width: @example-width-video;
    margin: 0 auto @padding-base;

    @media (min-width: @screen-sm) {
        margin-bottom: (@padding-base * 2);
    }
}