// ==========================================================================
// Plyr.io Demo Page
// ==========================================================================

// CSS Reset
@import "lib/normalize.less";

// Mixins
@import "lib/mixins.less";

// Variables
@import "variables.less";

// Animation
@import "lib/animation.less";

// Type
@import "lib/fontface.less";
@import "components/type.less";

// Components
@import "components/base.less";
@import "components/icons.less";
@import "components/buttons.less";
@import "components/error.less";
@import "components/examples.less";
