// ==========================================================================
// Fonts
// ==========================================================================

@font-face {
    font-family: "Avenir";
    src:    url("//cdn.plyr.io/fonts/avenir-medium.woff2") format("woff2"), 
            url("//cdn.plyr.io/fonts/avenir-medium.woff") format("woff");
    font-style:   normal;
    font-weight:  @font-weight-base;
}
@font-face {
    font-family: "Avenir";
    src:    url("//cdn.plyr.io/fonts/avenir-bold.woff2") format("woff2"), 
            url("//cdn.plyr.io/fonts/avenir-bold.woff") format("woff");
    font-style:   normal;
    font-weight:  @font-weight-bold;
}