// ==========================================================================
// Typography
// ==========================================================================

// Base
html {
    font-size: 100%;
}
body {
    font-family: "Avenir", "Helvetica Neue", Helvetica, Arial, sans-serif;
    line-height: 1.5;
    text-align: center;
    color: @gray;
    font-weight: @font-weight-base;
    .font-smoothing();
}

// Headings
h1,
h2 {
    letter-spacing: -.025em;
    color: @brand-primary;
    margin: 0 0 (@padding-base / 2);
    line-height: 1.2;
    font-weight: @font-weight-bold;
}
h1 {
    .font-size(@font-size-h1);
}

// Paragraph and small
p,
small {
    margin: 0 0 @padding-base;
}
small {
    display: block;
    padding: 0 (@padding-base / 2);
    .font-size(@font-size-small);
}

// Lists
ul,
li {
    list-style: none;
    margin: 0;
    padding: 0;
}

// Links
a {
    text-decoration: none;
    color: @link-color;
    border-bottom: 1px dotted currentColor;
    transition: background .3s ease, color .3s ease, border .3s ease;

    &:hover,
    &:focus {
        color: @gray-dark;
        border-bottom-color: rgba(0,0,0,0);
    }
    &:focus {
        .tab-focus();
    }
    &.logo {
        border: 0;
    }
}

.color--vimeo {
    color: @color-vimeo;
}
.color--youtube {
    color: @color-youtube;
}
