// ==========================================================================
// Examples
// ==========================================================================

// For non supported browsers
video {
    max-width: 100%;
    vertical-align: middle;
}

// Example players
.plyr {
    margin: 0 auto;
    border-radius: @border-radius-large;
}
.plyr--audio {
    max-width: @example-width-audio;
}
.plyr__video-wrapper::after {
    content: "";
    pointer-events: none;
    position: absolute;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    border: 1px solid fade(#000, 15%);
    border-radius: inherit;
}

// Style full supported player
.plyr__cite {
    display: none;
    margin-top: @padding-base;

    .icon {
        margin-right: (@padding-base / 4);
    }
}

.plyr--video:not(.plyr--youtube):not(.plyr--vimeo) ~ ul .plyr__cite--video,
.plyr--audio ~ ul .plyr__cite--audio,
.plyr--youtube ~ ul .plyr__cite--youtube,
.plyr--vimeo ~ ul .plyr__cite--vimeo {
    display: block;
}
